#include "Arduino.h"
#include "ULN2003.h"

// lets define the setup function
// ULN2003(p1,p2,p3,p4,steps per rev, step delay)
// Note: _stepdelay has to be an unsigned long to be able to avoid overflow of the timers thanks to register arithmetic
ULN2003::ULN2003(int _p1, int _p2, int _p3, int _p4, unsigned long _stepdelay, int _nRev)
{
  p1 = _p1;
  p2 = _p2;
  p3 = _p3;
  p4 = _p4;
  nRev = _nRev;
  pinMode(p1, OUTPUT);
  pinMode(p2, OUTPUT);
  pinMode(p3, OUTPUT);
  pinMode(p4, OUTPUT);
  phase = 0;
  steptime = _stepdelay;
  currentstep = 0;
  stepstogo = 0;
  readyToStop = 0;
}

// run() should be computationally cheap if false. Just check if we have to move
// (maybe checking objective vs current step? Or with a flag?) and send to the
// function. Or else return quickly so that we don't stall the loop.
boolean ULN2003::run()
{
  if (!stepstogo) // will be true if zero (!false = true)
  { //we're done
    if (readyToStop == 0)
    {
      readyToStop = 1;
      toStopTime = micros();
    }
    else if (readyToStop == 1)
    {
      unsigned long time = micros();

      if (time - toStopTime >= STOP_DELAY_TIME)
      {
        stop();
        readyToStop = 2;
      }
    }

    return true;
  }
  else {
    readyToStop = 0;
    nextStep();
  }
}

// Called because we still have to move a stepper. Do a time check to see if
// the next step is due and determine the sequence of phases that we need.
void ULN2003::nextStep()
{
  // Do time check first
  unsigned long time = micros();

  if (time - _lastStepTime >= steptime)  //this should work (not overflow), but test.
  {
    if (stepstogo > 0)
    { //clockwise
      stepCW();
      stepstogo--;
    }
    else if (stepstogo < 0)
    { //counter-clockwise
      stepCCW();
      stepstogo++;
    }
    _lastStepTime = time;
  }
}

// Setup a movement. Set stepstogo.
void ULN2003::move(long steps) {
  powerUp();
  if (nRev == 0)
  {
    stepstogo += steps;
  }
  else
  {
    stepstogo -= steps;
  }
}

// Inherits phase, calls for clockwise movement phase sequence
void ULN2003::stepCW()
{
  switch (phase) //gofromthisphase
  {
    case 0:
      goto7();
      break;
    case 1:
      goto0();
      break;
    case 2:
      goto1();
      break;
    case 3:
      goto2();
      break;
    case 4:
      goto3();
      break;
    case 5:
      goto4();
      break;
    case 6:
      goto5();
      break;
    case 7:
      goto6();
      break;
    default:
      goto0();
      break;
  }

  currentstep++;
}

// Inherits phase, calls for counter-clockwise movement phase sequence
void ULN2003::stepCCW()
{
  switch (phase) //gofromthisphase
  {
    case 0:
      goto1();
      break;
    case 1:
      goto2();
      break;
    case 2:
      goto3();
      break;
    case 3:
      goto4();
      break;
    case 4:
      goto5();
      break;
    case 5:
      goto6();
      break;
    case 6:
      goto7();
      break;
    case 7:
      goto0();
      break;
    default:
      goto0();
      break;
  }

  currentstep--;
}

#if FULLSTEP == 1
void ULN2003::goto1()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, LOW);
  digitalWrite(p3, HIGH);
  digitalWrite(p4, HIGH);
  phase = 1;
}
void ULN2003::goto2()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, HIGH);
  digitalWrite(p3, HIGH);
  digitalWrite(p4, LOW);
  phase = 2;
}
void ULN2003::goto3()
{
  digitalWrite(p1, HIGH);
  digitalWrite(p2, HIGH);
  digitalWrite(p3, LOW);
  digitalWrite(p4, LOW);
  phase = 3;
}
void ULN2003::goto4()
{
  digitalWrite(p1, HIGH);
  digitalWrite(p2, LOW);
  digitalWrite(p3, LOW);
  digitalWrite(p4, HIGH);
  phase = 4;
}
void ULN2003::goto5()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, LOW);
  digitalWrite(p3, HIGH);
  digitalWrite(p4, HIGH);
  phase = 5;
}
void ULN2003::goto6()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, HIGH);
  digitalWrite(p3, HIGH);
  digitalWrite(p4, LOW);
  phase = 6;
}
void ULN2003::goto7()
{
  digitalWrite(p1, HIGH);
  digitalWrite(p2, HIGH);
  digitalWrite(p3, LOW);
  digitalWrite(p4, LOW);
  phase = 7;
}
void ULN2003::goto0()
{
  digitalWrite(p1, HIGH);
  digitalWrite(p2, LOW);
  digitalWrite(p3, LOW);
  digitalWrite(p4, HIGH);
  phase = 0;
}
#else
void ULN2003::goto1()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, LOW);
  digitalWrite(p3, LOW);
  digitalWrite(p4, HIGH);
  phase = 1;
}
void ULN2003::goto2()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, LOW);
  digitalWrite(p3, HIGH);
  digitalWrite(p4, HIGH);
  phase = 2;
}
void ULN2003::goto3()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, LOW);
  digitalWrite(p3, HIGH);
  digitalWrite(p4, LOW);
  phase = 3;
}
void ULN2003::goto4()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, HIGH);
  digitalWrite(p3, HIGH);
  digitalWrite(p4, LOW);
  phase = 4;
}
void ULN2003::goto5()
{
  digitalWrite(p1, LOW);
  digitalWrite(p2, HIGH);
  digitalWrite(p3, LOW);
  digitalWrite(p4, LOW);
  phase = 5;
}
void ULN2003::goto6()
{
  digitalWrite(p1, HIGH);
  digitalWrite(p2, HIGH);
  digitalWrite(p3, LOW);
  digitalWrite(p4, LOW);
  phase = 6;
}
void ULN2003::goto7()
{
  digitalWrite(p1, HIGH);
  digitalWrite(p2, LOW);
  digitalWrite(p3, LOW);
  digitalWrite(p4, LOW);
  phase = 7;
}
void ULN2003::goto0()
{
  digitalWrite(p1, HIGH);
  digitalWrite(p2, LOW);
  digitalWrite(p3, LOW);
  digitalWrite(p4, HIGH);
  phase = 0;
}
#endif

// Returns current step
int ULN2003::currentPosition()
{
  return currentstep;
}

// Returns steps to go
int ULN2003::stepsToGo()
{
  return stepstogo;
}

// Pulls pins low to save power and avoid heat build up when not moving.
void ULN2003::stop()
{
  stepstogo = 0;
  digitalWrite(p1, LOW);
  digitalWrite(p2, LOW);
  digitalWrite(p3, LOW);
  digitalWrite(p4, LOW);
}

// Powers pins up again at current phase to get ready to move. This prevents
// stalling and improves the response to CCW movement, allowing faster speeds
// (lower step delays)
void ULN2003::powerUp()
{
  switch (phase) //gofromthisphase
  {
    case 0:
      goto0();
      break;
    case 1:
      goto1();
      break;
    case 2:
      goto2();
      break;
    case 3:
      goto3();
      break;
    case 4:
      goto4();
      break;
    case 5:
      goto5();
      break;
    case 6:
      goto6();
      break;
    case 7:
      goto7();
      break;
    default:
      goto0();
      break;
  }
}
